import os

import pyPman
from pyPman.utilities import utils
from pyPman.install.utils import CommandInitializer


class InitCommandGenerator(CommandInitializer):
    def __init__(self, *args):
        super(InitCommandGenerator, self).__init__(*args)

    @property
    def assigner(self):
        return [
            "__version__ = '%s'" % self.package_dict['version'],
            "__name__ = '%s'" % self.package_dict['name'],
            "__author__ = ['%s']" % self.package_dict['author'],
            "__author_email__ = ['%s']" % self.package_dict['authorMail']
        ]


def command_builder(package_dict):
    cmd_gen = InitCommandGenerator(package_dict)
    init_file = utils.build_cmd_file(cmd_gen.initializer, script_list=[], class_or_def=False)
    init_file = utils.build_cmd_file(cmd_gen.assigner, script_list=init_file, class_or_def=False)
    init_file.append("")
    final_init_file = "\n".join(init_file)
    return final_init_file



import sys
import os as _os
import logging
import pprint
import json

from traVer import Version

log = logging.getLogger(__name__)

PROJECT_NAME = None
PACKAGE_NAME = None
PACKAGE_VERSION = "1.0.0"
DESCRIPTION = None
ENTRY_POINT = "__main__.py"
GIT_REPO = None
KEYWORDS = None
AUTHOR = None
AUTHOR_EMAIL = None
LICENSE = "GNU V3"


def create_files(output, file_name, data=None):
    if not _os.path.exists(output):
        log.error("Please make sure that the output directory exists")
        return False
    file_path = _os.path.join(output, file_name)
    with open(file_path, 'w') as newFile:
        if data and isinstance(data, dict):
            json.dump(data, newFile, indent=2)
        elif data:
            newFile.write(data)
    log.info("%s file created in %s" % (file_name, output))
    return True


def get_project_details(install_path, pkg_file=None):
    package_file = ".%s.json" % (pkg_file or PROJECT_NAME)
    pkg_file_path = _os.path.join(install_path, package_file)
    if check_for_project(install_path, pkg_file=pkg_file):
        with open(pkg_file_path) as fd:
            data = json.load(fd)
        return data


def check_for_project(install_path, pkg_file=None):
    package_file = ".%s.json" % (pkg_file or PROJECT_NAME)
    pkg_file_path = _os.path.join(install_path, package_file)
    if _os.path.exists(pkg_file_path):
        return True
    return False


def init_project(install_path, project=None):
    global PROJECT_NAME, PACKAGE_NAME, PACKAGE_VERSION, DESCRIPTION, ENTRY_POINT, GIT_REPO
    global KEYWORDS, AUTHOR, AUTHOR_EMAIL, LICENSE
    pkg_name = _os.path.basename(install_path)
    package_dict = {}
    if project:
        log.info("Project name set to: %s" % project)
        query_detail("Project Name", default=project, accepted=True)
    PROJECT_NAME = project or query_detail("Project Name", default=pkg_name)
    if check_for_project(install_path):
        package_dict = get_project_details(install_path)
        log.warning("%s already been setup in the directory" % PROJECT_NAME)
    PACKAGE_NAME = query_detail("Package Name", default=PROJECT_NAME)
    PACKAGE_VERSION = query_detail("Project Version", default=package_dict.get('version', None) or PACKAGE_VERSION)
    try:
        pkg_version = Version(PACKAGE_VERSION)
    except Exception as err:
        log.error("%s, Please use semantic versioning syntax i.e., 12.3.4 (or) 2.3.4-beta.5" % err)
        sys.exit()

    DESCRIPTION = query_detail("Description", default=package_dict.get('description', None))
    ENTRY_POINT = query_detail("Entry Point", default=package_dict.get('main', None) or ENTRY_POINT)
    GIT_REPO = query_detail("Git Repository", default=package_dict.get('url', None))
    KEYWORDS = query_detail("Keywords", default=package_dict.get('keywords', None))
    AUTHOR = query_detail("Author", default=package_dict.get('author', None))
    AUTHOR_EMAIL = query_detail("Author Email", default=package_dict.get('authorMail', None))
    LICENSE = query_detail("License", default=package_dict.get('license', None) or LICENSE)

    package_dict = {
        "name": PROJECT_NAME,
        "package": PACKAGE_NAME,
        "version": PACKAGE_VERSION,
        "description": DESCRIPTION,
        "main": ENTRY_POINT,
        "author": AUTHOR,
        "authorMail": AUTHOR_EMAIL,
        "license": LICENSE,
        "url": GIT_REPO,
        "dependency": package_dict.get("dependency", []),
        "module": package_dict.get("module", []),
    }

    log.warning("About to write the following dictionary to <package>.json\n")
    pprint.pprint(package_dict)

    accept = query_detail("Is this Okay?", default="yes")
    if not accept == "yes":
        log.warning("Project creation exited by user.")
        sys.exit()

    create_files(install_path, ".%s.json" % PROJECT_NAME, data=package_dict)
    return package_dict


def query_detail(query, default=None, accepted=False):
    query = query.strip() + ": "
    if default:
        query = query + "(%s) " % default
    if accepted:
        print(query)
        return
    result = input(query)

    if result:
        return result
    else:
        return default



"""
    Main entry point for the application to start
"""
import sys
import os as _os
import click
import logging

from pyPman import __version__
from pyPman.initialize import projectInitialize as pInit
from pyPman.install import builder as p_build
from pyPman.utilities import utils
from pyPman.utilities import moduleVersion

OUTPUT_DIR = _os.getcwd()
log = logging.getLogger(__name__)


class ListParamType(click.ParamType):
    name = 'list'

    def convert(self, value, param, ctx):
        return value.split('==')


@click.group()
@click.version_option(version=__version__, prog_name="PyProjectManager")
@click.help_option("-h", "--help", help="Displays this help options and exits")
@click.option('--verbose', is_flag=True, help='Displays all the log.')
@click.pass_context
def cli(context, verbose):
    if verbose:
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.WARNING)


@cli.command()
@click.pass_context
@click.help_option("-h", "--help", help="Displays this help options and exits")
@click.option("--project", "-p", type=str, default=None, help="Project name")
@click.option("--dependency", "-d", type=ListParamType(), multiple=True,
              help="Add dependency projects to the existing project with module name and version(module_name==version")
@click.option("--module", '-m', type=str, multiple=True, help="Add new module to the project")
def init(context, project, dependency, module):
    logger = """
This utility will walk you through creating a setup.py file. It only covers the most common items, and tries to guess sensible defaults.

See `pyPman help install` for definitive documentation on these fields and exactly what they do.

Use `pyPman install -p <projectName>` afterwards to install a package and save it as a dependency in the setup.py file.

Press ^C at any time to quit.
    """
    if dependency:
        if not project:
            log.warning("Please specify the project name to add dependency by adding --project to command")
            return

        package_dict = {}
        if pInit.check_for_project(OUTPUT_DIR, pkg_file=project):
            package_dict = pInit.get_project_details(OUTPUT_DIR, pkg_file=project)
        if not package_dict:
            log.warning("Unable to find the project {0}, Please run `pyPman init --project {0}` to add dependency".format(
                project
            )
            )
            return
        is_updated = False
        for depend in dependency:
            log.info("Adding dependency for {0}".format(depend[0]))
            if len(depend) > 2:
                log.error("Arguments for dependency must be either <Dependency>==<version> or only <Dependency>")
                sys.exit()
            dep = False
            if utils.check_for_dependency(depend, package_dict):
                dep = True
                if utils.check_for_dep_version(depend, package_dict):
                    log.warning("Dependency '%s' already added to the project" % depend[0])
                    continue
            if len(depend) == 2:
                if dep:
                    for i, module in enumerate(package_dict['dependency']):
                        if module[0] == depend[0]:
                            package_dict['dependency'].pop(i)
                            is_updated = True
                            break

                m_version = moduleVersion.get_module_version(depend[0], module_version=depend[1])
                if not m_version:
                    # click.echo("Unable to find the module/module version. Exiting...")
                    log.warning("Unable to find the {0} or {1} version.".format(depend[0], depend[1]))
                    continue
                    # sys.exit()
                else:
                    package_dict['dependency'].append(depend)
                    is_updated = True
            elif len(depend) == 1:
                m_version = moduleVersion.get_module_version(depend[0])
                package_dict['dependency'].append([depend[0], m_version])
                is_updated = True
            click.echo("Dependency updated with %s module" % depend[0])
        if is_updated:
            pInit.create_files(OUTPUT_DIR, ".%s.json" % project, data=package_dict)
        if not module:
            return

    if module:
        if not project:
            log.warning("Please specify the project name to add dependency by adding --project to command")
            return
        package_dict = {}
        if pInit.check_for_project(OUTPUT_DIR, pkg_file=project):
            package_dict = pInit.get_project_details(OUTPUT_DIR, pkg_file=project)
        if not package_dict:
            log.warning("Unable to find the project {0}, Please run `pyPman init --project {0}` to add dependency"
                .format(project)
            )
            return
        for mod in module:
            if utils.check_for_module(mod, package_dict):
                log.warning("Module '%s' already exists in the project" % mod)
                continue
            package_dict['module'].append(mod)
            pInit.create_files(OUTPUT_DIR, ".%s.json" % project, data=package_dict)
            click.echo("Module '%s' added to project" % mod)
        return

    click.echo(message=logger)
    pInit.init_project(OUTPUT_DIR, project=project)


@cli.command()
@click.pass_context
@click.help_option("-h", "--help", help="Displays this help options and exits")
@click.option("--project", "-p", type=str, default=None, help="Project name")
# @click.option("project", type=str, help="Project name", required=True)
def install(context, project):
    click.echo(message="Starting installing from the saved data")
    package_dict = {}
    if pInit.check_for_project(OUTPUT_DIR, pkg_file=project):
        package_dict = pInit.get_project_details(OUTPUT_DIR, pkg_file=project)
    if not package_dict:
        log.warning("Unable to find the project {0}, Please run `pyPman init --project {0}` to add dependency".format(
                project
            )
        )
        return
    p_build.install(OUTPUT_DIR, package_dict)
    click.echo(message="%s project installed in %s directory" %(project, OUTPUT_DIR))


@cli.command()
@click.pass_context
@click.argument("option")
@click.help_option("-h", "--help", help="Displays this help options and exits")
def help(context, option):
    options = ['init', 'install']
    if not option in options:
        log.error("No option called '%s' in pyPman" % option)
        sys.exit()
    click.echo("Displays help for the option given %s" % option)

    helper_msg = ""
    if option == "init":
        helper_msg = """
Complete help information about init:
    * pyPman init : Starts project initialization
    * pyPman init [-p <projectName>] : Starts project initialization setting projectName as provided
    * pyPman init [-p <projectName> -d <dependancy>] : Adds 3rd party dependancy modules to the project
        Usage: pyPman init -p <projectName> -d <moduleName>[==<module_version>]....
                Mentioning no version will take the latest version of specified dependency
            * Ex: pyPman init -p PyProjectManager -d click (or)
            * Ex: pyPman init -p PyProjectManager -d click==7.1.1
            * Ex: pyPman init -p PyProjectManager -d click==7.1.1 -d traVer...(For multiple dependencies
    * pyPman init [-p <projectName> -m <module>] : Adds user defined module package to the project
        Usage: pyPman init -p <projectName> -m <moduleName>
            * Ex: pyPman init -p PyProjectManager -m testModule 
            * Ex: pyPman init -p PyProjectManager -m testModule1 -m testModule2 ...
        """

    elif option == "install":
        helper_msg = """
Complete help information about install:
    * pyPman install [-p <projectName>] : Starts installing project to the current directory
        Usage: pyPman install -p <projectName>
            * Ex: pyPman install -p PyProjectManager
        """

    click.echo(helper_msg)



from pyPman.install import utils as install_utils
from pyPman.utilities import utils


class CommandGenerator(install_utils.CommandInitializer):
    def __init__(self, *args):
        super().__init__(*args)

    @property
    def license_creator(self):
        default_license = ""
        main_license = None
        if self.package_dict['license'] == "GNU V3":
            main_license = default_license
        else:
            # TODO: Later options may provide the license availability for other licenses
            print("Please copy your license in 'LICENSE' file if available")
        return [
            "%s" % main_license
        ]


def command_builder(package_dict):
    cmd_gen = CommandGenerator(package_dict)
    main_file = utils.build_cmd_file(cmd_gen.initializer, class_or_def=False)
    main_file = utils.build_cmd_file(cmd_gen.readme_creator, script_list=main_file)
    main_file.append("")
    return "\n".join(main_file)


class CommandInitializer(object):
    def __init__(self, pkg_dict):
        self.package_dict = pkg_dict

    @property
    def initializer(self):
        return [
            "# This project is build with %s %sV" % (self.package_dict['name'], self.package_dict['version']),
            "# Please don't edit without knowing much about the working project",
            "# If you have any queries about python project builder,",
            "# Please contact below person(s)\n# %s - %s" % (
                self.package_dict['author'], self.package_dict['authorMail']
            )
        ]



import sys
import subprocess
from importlib_metadata import version, PackageNotFoundError

def get_module_version(module, module_version=None, tries=0):
    if module_version:
        return module_version
    try:
        return version(module)
    except PackageNotFoundError:
        print("%s Trying to install '%s'..." % (tries + 1, module))
        if module_version:
            subprocess.call([sys.executable, '-m', 'pip', 'install', '{0}=={1}'.format(module, module_version)])
        else:
            subprocess.call([sys.executable, '-m', 'pip', 'install', '{0}'.format(module)])
        if tries == 2:
            print("Unable to find the package '%s' from PyPi, Please ensure before adding" % module)
            return
        tries += 1
        return get_module_version(module, module_version, tries=tries)
    except:
        print("Unable to find the issue, please contact sunil.nerella39@gmail.com for solution")




"""
Build structure:
    projectName
        \____ packageName
        \       \____ __init__.py
        \       \____ __main__.py(entry point)
        \____ README.md
        \____ LICENSE
        \____ setup.py

    By building the project with PyProject manager, you can use the project for installing using 'pip'

"""
import os
from pyPman.install import setup_builder, init_builder, main_builder, readme_builder, license_builder
from pyPman.utilities import utils


def setup_structure(output, package_dict):
    """ Definition to setup the directory and files structure in output folder
    """
    directories = [
        package_dict['name'],
        os.path.join(package_dict['name'], package_dict['package'])
    ]
    user_modules = [os.path.join(
        package_dict['name'],
        package_dict['package'],
        module
    ) for module in package_dict['module']]
    directories.extend(user_modules)

    files = [
        os.path.join(package_dict['name'], "README.md"),
        os.path.join(package_dict['name'], "LICENSE"),
        os.path.join(package_dict['name'], "setup.py"),
        os.path.join(package_dict['name'], os.path.join(package_dict['package'], "__init__.py")),
        os.path.join(package_dict['name'], os.path.join(package_dict['package'], package_dict['main']))
    ]
    user_module_initialise = [os.path.join(module_dir, '__init__.py') for module_dir in user_modules]
    files.extend(user_module_initialise)

    for directory in directories:
        utils.create_directory(output, directory)

    for file_name in files:
        utils.create_file(output, file_name=file_name)

    return files


def install(output, package_dict):
    # Setting up the file structure in the output directory
    script_files = setup_structure(output, package_dict)

    # Generating file scripts
    init_build_file = init_builder.command_builder(package_dict)
    main_build_file = main_builder.command_builder(package_dict)
    setup_build_file = setup_builder.commands(package_dict)
    print("Writing new files to the scripts")
    utils.write_file(script_files[3], data=init_build_file)
    utils.write_file(script_files[4], data=main_build_file)
    utils.write_file(script_files[2], data=setup_build_file)



from pyPman.utilities import utils
from pyPman.install import utils as install_utils


class CommandGenerator(install_utils.CommandInitializer):
    def __init__(self, *args):
        super().__init__(*args)

    @property
    def readme_creator(self):
        return [
            "# %s" % self.package_dict['name'],
            "%s" % self.package_dict['description']
        ]


def command_builder(package_dict):
    cmd_gen = CommandGenerator(package_dict)
    main_file = utils.build_cmd_file(cmd_gen.initializer, class_or_def=False)
    main_file = utils.build_cmd_file(cmd_gen.readme_creator, script_list=main_file)
    main_file.append("")
    return "\n".join(main_file)



from pyPman.utilities import utils
from pyPman.install import utils as install_utils


class MainCommandGenerator(install_utils.CommandInitializer):
    def __init__(self, *args):
        super(MainCommandGenerator, self).__init__(*args)

    @property
    def cli_creator(self):
        return [
            "def cli():",
            "   # Add your code here to run the command line interface",
            "   print('Command line interface will exit after displaying this message.')",
            "   print('If you want to edit, go to <projectName>/<packageName>/__main__.py')",
            "",
            "",
            "def cli_ui():",
            "   # Add your code here to run the command line interface",
            "   pass"

        ]


def command_builder(package_dict):
    cmd_gen = MainCommandGenerator(package_dict)
    main_file = utils.build_cmd_file(cmd_gen.initializer, script_list=[], class_or_def=False)
    main_file = utils.build_cmd_file(cmd_gen.cli_creator, script_list=main_file)
    main_file.append("")
    final_main_file = "\n".join(main_file)
    return final_main_file



from setuptools import setup
import os
from glob import glob
from shutil import rmtree

packageName = "pyPman"
try:
    from setuptools.command.clean import clean as clean_command
except ImportError as err:
    from distutils.command.clean import clean as clean_command


class Clean(clean_command):
    def run(self):
        clean_command.run(self)
        delete_in_root = ['build', 'dist', '*.egg-info', '.tox', '.eggs', '.cache']
        delete_everywhere = ['__pycache__', '*.pyc']

        for candidate in delete_in_root:
            rmtree_glob(candidate)

        for visible_dir in glob("[A-Za-z0-9]*"):
            for candidate in delete_everywhere:
                rmtree_glob(os.path.join(visible_dir, candidate))
                rmtree_glob(os.path.join(visible_dir, "*", candidate))


def rmtree_glob(candidate):
    for fobj in glob(candidate):
        try:
            rmtree(fobj)
            print("%s/ removed from project" % fobj)
        except OSError:
            try:
                os.remove(fobj)
                print("%s/ removed from project" % fobj)
            except OSError:
                print("Testing")


def read_file(path=None, file_name=None):
    path = path or "."
    file_path = os.path.join(path, file_name)
    print("Reading file %s" % file_path)
    with open(file_path) as fd:
        return fd.read()


exec(read_file(path=os.path.join('.', packageName), file_name="__init__.py"))
version = __version__
projectName = __name__
readmeData = read_file(file_name="README.md")
licenseData = read_file(file_name="LICENSE")

setup(
    name=packageName,
    description="Python project manager",
    version=version,
    packages=[
        packageName,
        "%s.initialize" % packageName,
        "%s.install" % packageName,
        "%s.utilities" % packageName,
    ],
    install_requires=[
        "Click",
        "traVer == 1.1.0",
        "setuptools",
        "importlib_metadata"
    ],
    license="GNU GENERAL PUBLIC LICENSE",
    long_description_content_type='text/markdown',
    long_description=readmeData,
    author=', '.join(__author__),
    author_email=', '.join(__author_email__),
    url="https://github.com/sunilk-n/TraVersion",
    classifiers=[
        # See https://pypi.org/pypi?%3Aaction=list_classifiers
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    data_files=[
        "README.md",
        "LICENSE"
    ],
    entry_points={
        'console_scripts': [
            'pyPman=pyPman.__main__:cli'
        ],
    },
    cmdclass={
        "clean": Clean,
    },
)



import os
from pyPman.utilities import utils
from pyPman.install import utils as install_utils


class SetupCommandGenerator(install_utils.CommandInitializer):
    def __init__(self, *args):
        super(SetupCommandGenerator, self).__init__(*args)

    @property
    def importer(self):
        return [
            "import os",
            "import sys",
            "from glob import glob",
            "from shutil import rmtree",
            "from setuptools import setup",
            "",
            "try:",
            "   from setuptools.command.clean import clean as clean_command",
            "except ImportError as err:",
            "   from distutils.command.clean import clean as clean_command",
        ]

    @property
    def cleanClass(self):
        return [
            "class Clean(clean_command):",
            "   def run(self):",
            "       clean_command.run(self)",
            "       delete_in_root = ['build', 'dist', '*.egg-info', '.tox', '.eggs', '.cache']",
            "       delete_everywhere = ['__pycache__', '*.pyc']",
            "",
            "       for candidate in delete_in_root:",
            "           rmtree_glob(candidate)",
            "",
            "       for visible_dir in glob('[A-Za-z0-9]*'):",
            "           for candidate in delete_everywhere:",
            "               rmtree_glob(os.path.join(visible_dir, candidate))",
            "               rmtree_glob(os.path.join(visible_dir, '*', candidate))",
        ]

    @property
    def globRmtreeDef(self):
        return [
            "def rmtree_glob(candidate):",
            "   for fobj in glob(candidate):",
            "       try:",
            "           rmtree(fobj)",
            "           print('%s/ removed from project' % fobj)",
            "       except OSError:",
            "           try:",
            "               os.remove(fobj)",
            "               print('%s/ removed from project' % fobj)",
            "           except OSError:",
            "               print('Unable to clean the %s/ from project' % fobj)",
        ]

    @property
    def readFileDef(self):
        return [
            "def read_file(path=None, file_name=None):",
            "   path = path or '.'",
            "   file_path = os.path.join(path, file_name)",
            "   print('Reading file %s' % file_path)",
            "   with open(file_path) as fd:",
            "       return fd.read()",
        ]

    @property
    def mainExecution(self):
        return [
            # TODO: Change package name once everything is completed
            "packageName = '%s'" % self.package_dict['package'],
            "exec(read_file(path=os.path.join('.', packageName), file_name='__init__.py'))",
            "version = __version__",
            "projectName = __name__",
            "readmeData = read_file(file_name='README.md')",
            "licenseData = read_file(file_name='LICENSE')",
        ]

    @property
    def basicSetup(self):
        dependencyStr = ", ".join([
            "'%s'" % "==".join(module) if module[1] else "'%s'" % module[0] for module in self.package_dict['dependency']
        ])
        packageStr = ', '.join(
            [
                # "%s.initialize" % packageName
                '"%s.{}" % packageName'.format(pkg) for pkg in self.package_dict['module']
            ]
        )
        return [
            "setup(",
            "   name=packageName,",
            # TODO: Change this
            "   description='%s'," % self.package_dict['description'],
            "   version=version,",
            "   packages=[",
            "       packageName,",
            "       # Add modules under your package to build while installing your module",
            "       # Like packageName.utilities, <your packageName>.<module under your package>",
            "       %s" % packageStr,
            "   ],",
            "   install_requires=[",
            # TODO: Add dependency modules
            "       %s" % dependencyStr,
            "       # Add your project requirements (External Modules) here",
            "   ],",
            # TODO: Assign license here
            "   license='%s'," % self.package_dict['license'],
            "   long_description_content_type='text/markdown',",
            "   long_description=readmeData,",
            "   author=', '.join(__author__),",
            "   author_email=', '.join(__author_email__),",
            # TODO: Assign url if exists
            "   url='%s'," % self.package_dict['url'] if self.package_dict['url'] else "",
            "   classifiers=[",
            "   # See https://pypi.org/pypi?%3Aaction=list_classifiers",
            "       '',",
            "   ],",
            "   python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',",
            "   data_files=[",
            "       'README.md',",
            "       'LICENSE',",
            "   ],",
            "   entry_points={",
            # TODO: Fill entry point if needed
            "       'console_scripts': [",
            "           # Change the <cli_cmd> to your preferred name",
            "           'cli_cmd=%s.%s:cli'," % (
                            self.package_dict['package'],
                            os.path.splitext(self.package_dict['main'])[0]
                        ),
            "       ],",
            "       'gui_scripts': [",
            "           # Change the <cli_cmd_ui> to your preferred name or",
            "           # remove this 'gui_scripts' keyValue pair if gui app is not required for you module",
            "           'cli_cmd_ui=%s.%s:cli_ui'," % (
                            self.package_dict['package'],
                            os.path.splitext(self.package_dict['main'])[0]
                        ),
            "       ],",
            "   },",
            "   cmdclass={",
            "       'clean': Clean",
            "   },",
            ")",
        ]


def commands(package_dict):
    cmd_gen = SetupCommandGenerator(package_dict)
    setup_file = utils.build_cmd_file(cmd_gen.initializer, script_list=[], class_or_def=False)
    setup_file = utils.build_cmd_file(cmd_gen.importer, script_list=setup_file, class_or_def=False)
    setup_file = utils.build_cmd_file(cmd_gen.cleanClass, script_list=setup_file)
    setup_file = utils.build_cmd_file(cmd_gen.globRmtreeDef, script_list=setup_file)
    setup_file = utils.build_cmd_file(cmd_gen.readFileDef, script_list=setup_file)
    setup_file = utils.build_cmd_file(cmd_gen.mainExecution, script_list=setup_file)
    setup_file = utils.build_cmd_file(cmd_gen.basicSetup, script_list=setup_file)
    setup_file.append('')
    return "\n".join(setup_file)



import os
import click
import logging

from pyPman.initialize import projectInitialize as pInit

log = logging.getLogger(__name__)


class ModuleSelector(click.Option):
    def __init__(self, *args, **kwargs):
        nargs = kwargs.pop('nargs', -1)
        assert nargs == -1, 'nargs, if set, must be -1 not {}'.format(nargs)
        super(ModuleSelector, self).__init__(*args, **kwargs)
        self._module_parser = None

    def add_to_parser(self, parser, ctx):
        def parser_process(value, state):
            done = False
            value = [value]
            value += state.rargs
            state.rargs[:] = []
            value = tuple(value)


def check_for_dependency(dependency, package_dict):
    module = dependency[0]
    for savedModule in package_dict['dependency']:
        if module == savedModule[0]:
            return True
    return False


def check_for_dep_version(dependency, package_dict):
    module = dependency[0]
    m_version = None
    if len(dependency) == 2:
        m_version = dependency[1]
    if not m_version:
        return True
    for savedModule in package_dict['dependency']:
        if module == savedModule[0]:
            if m_version == savedModule[1]:
                return True
    return False


def check_for_module(module, package_dict):
    for added_module in package_dict['module']:
        if added_module == module:
            return True
    return False


def create_directory(output, dir_name):
    dir_path = os.path.join(output, dir_name)
    if os.path.exists(dir_path):
        log.warning("Directory name already exists.")
        return

    os.makedirs(dir_path)
    log.info("%s directory created in %s" % (dir_name, output))


def create_file(output, file_name="__init__.py"):
    if not os.path.exists(output):
        log.warning("Directory doesn't exists, Creating the directory")
        dir_path = os.path.dirname(output)
        dir_name = os.path.basename(output)
        create_directory(dir_path, dir_name)

    file_path = os.path.join(output, file_name)

    if os.path.exists(file_path):
        log.warning("Filename %s already exists in %s" % (file_name, output))
        return

    with open(file_path, 'w') as fd:
        fd.write("# %s created with PyProjectManager, you can remove this line later" % file_name)
    log.info("%s file created in %s" % (file_name, output))


def write_file(file_path, data=None):
    if not os.path.exists(file_path):
        dir_path = os.path.dirname(file_path)
        file_name = os.path.basename(file_path)
        create_file(dir_path, file_name=file_name)
    with open(file_path, 'w') as fd:
        fd.write(data)
    log.info("Data written into file %s" % file_path)


def build_cmd_file(list_of_commands, script_list=[], class_or_def=True):
    if class_or_def:
        script_list.extend(["", ""])
    else:
        script_list.append("")
    script_list.extend(list_of_commands)
    return script_list
